# @Author: Tushar(tusharcoder) <tushar>
# @Date:   05/09/18
# @Email:  tamyworld@gmail.com
# @Filename: fields
# @Last modified by:   Tushar

import re
import binascii

from django.core import validators
from django.db.models.fields import Field
from django.core.exceptions import ValidationError


HEXADECIMAL_VALUES = re.compile(r'^[0-9a-fA-F]+\Z')
try:
    basestring  # Python 3 has no basestring
except NameError:
    basestring = str


class BinaryHashField(Field):
    description = 'Hash data saved as binary'
    empty_values = (None, '')

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = self.hex_length
        super(BinaryHashField, self).__init__(*args, **kwargs)
        self.validators.append(validators.MaxLengthValidator(self.max_length))

    def get_internal_type(self):
        return "BinaryField"

    def hex_to_bytes(self, value):
        if value is None:
            return value
        elif isinstance(value, basestring) \
                and len(value) == self.hex_length \
                and HEXADECIMAL_VALUES.search(str(value)):
            return binascii.unhexlify(value)
        else:
            message_tpl = 'Enter a valid {alg} (hexadecimal string with {size} bytes).'
            message = message_tpl.format(alg=self.algorithm, size=self.hex_length)
            raise ValidationError(message)

    def get_db_prep_value(self, value, connection, prepared=False):
        value = super(BinaryHashField, self).get_db_prep_value(value, connection, prepared)
        if value not in self.empty_values:
            return connection.Database.Binary(self.hex_to_bytes(value))

    def from_db_value(self, value, expression, connection, context):
        if value not in self.empty_values:
            return binascii.hexlify(value).decode('ascii')

    def to_python(self, value):
        self.hex_to_bytes(value)
        return value

    def get_default(self):
        if self.has_default() and not callable(self.default):
            self.hex_to_bytes(self.default)
            return self.default
        default = super(BinaryHashField, self).get_default()
        return default

    def formfield(self, **kwargs):
        return super(BinaryHashField, self).formfield(max_length=self.max_length)

class SHA1Field(BinaryHashField):
    description = 'SHA-1 hash data saved as binary'
    algorithm = 'SHA-1'
    hex_length = 40





# @Author: Tushar(tusharcoder) <tushar>
# @Date:   07/09/18
# @Email:  tamyworld@gmail.com
# @Filename: app_settings
# @Last modified by:   Tushar

from django.conf import settings
try:
    SEND_REGISTRATION_MAILS = settings.SEND_REGISTRATION_MAILS
except:
    SEND_REGISTRATION_MAILS = True


# @Author: Tushar(tusharcoder) <tushar>
# @Date:   05/09/18
# @Email:  tamyworld@gmail.com
# @Filename: urls
# @Last modified by:   Tushar

from django.conf.urls import url

from .views import *

urlpatterns = [
    url(r'^request/forgot/password/$', forgot_password_view, name='request_forgot_password'),
    url(r'^check/forgot_password_code/$', forgot_password_confirm, name='check_forgot_password_code'),
    url(r'^reset/password/$', reset_password, name='reset_password'),
    url(r'^change/password/$', change_password, name='change_password'),
    #login user
    url(r'^login/$', login, name='login'),
    #register user
    url(r'^register/$', registration_view, name='register_user'),
]



# @Author: Tushar(tusharcoder) <tushar>
# @Date:   05/09/18
# @Email:  tamyworld@gmail.com
# @Filename: utils
# @Last modified by:   Tushar
import hashlib
from random import randint

from django.conf.global_settings import DEFAULT_FROM_EMAIL


def validate(*args, **kwargs):
    """function to validate the data in kwargs"""
    fields_to_check = args
    errors = []
    for field in fields_to_check:
        if kwargs.get(field) is None or kwargs.get(field).strip() == '':
            errors.append("{} is required and cannot be empty".format(field))
    return not len(errors) > 0, errors

def hexify(val):
    """function to return the hex of the value"""
    return hashlib.sha1(val.encode('utf8')).hexdigest()

def gen_hash():
    """return the random hash string of 6 values"""
    return hashlib.sha1(str(randint(000000,999999)).encode('utf8')).hexdigest()[-6:]

def send_mail(*args,**kwargs):
    """function to send the pdf attachment to the emails
        requires subject, body, html_body(optional), to_email
    """
    try:
        from django.core.mail import EmailMultiAlternatives
        to = kwargs.pop('to_email')
        if not isinstance(to, list):
            if isinstance(to, str):
                to = [to,]
        subject, from_email, to = kwargs.pop('subject'), DEFAULT_FROM_EMAIL, to
        text_content = kwargs.pop('body')
        if 'html_body' in kwargs:
            html_content = kwargs.pop('html_body')
        else:
            html_content = None
        msg = EmailMultiAlternatives(subject, text_content, from_email, to)
        if html_content:
            msg.attach_alternative(html_content, 'text/html')
        msg.send(fail_silently=True)
    except:
        pass

def get_request_data(request):
    return {key: request.data.get(key) for key in request.data.keys()}


from django.contrib.auth import authenticate
from django.contrib.auth.models import User
from django.db import IntegrityError
from rest_framework import status
from rest_framework.authentication import TokenAuthentication
from rest_framework.authtoken.models import Token
from rest_framework.decorators import api_view, authentication_classes, permission_classes
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from .models import ForgotPasswordModel
from .utils import validate, gen_hash, hexify, send_mail, get_request_data

from .app_settings import SEND_REGISTRATION_MAILS


# Create your views here.
@api_view(['POST', ])
def forgot_password_view(request):
    if request.method == "POST":
        data = get_request_data(request)
        valid, errors = validate(*['username'], **get_request_data(request))
        if not valid:
            return Response({"error": errors}, status=status.HTTP_422_UNPROCESSABLE_ENTITY)
        username = data.get("username")
        user = User.objects.get(username=username)
        # hash code
        code = gen_hash()
        hex_code = hexify(code)
        ForgotPasswordModel.objects.create(code=hex_code, user=user)
        send_mail(
            **{"body": "Reset password code is {}".format(code), "subject": "Forgot Password", "to_email": user.email})
        return Response({'message': ['Reset code generated', ], 'code': code}, status=status.HTTP_201_CREATED)
    else:
        return Response({'error': ['{} method is not allowed'.format(request.method)]},
                        status=status.HTTP_405_METHOD_NOT_ALLOWED)


@api_view(['POST', ])
def forgot_password_confirm(request):
    if request.method == "POST":
        valid, errors = validate(*['code'], **get_request_data(request))
        if not valid:
            return Response({"error": errors}, status=status.HTTP_422_UNPROCESSABLE_ENTITY)
        data = get_request_data(request)
        hex_code = hexify(data.get("code"))
        exists = ForgotPasswordModel.objects.filter(code=hex_code).exists()
        if exists:
            return Response({"message": ['code exists']}, status=status.HTTP_200_OK)
        else:
            return Response({"error": ['code invalid']}, status=status.HTTP_404_NOT_FOUND)
    else:
        return Response({'error': ['{} method is not allowed'.format(request.method)]},
                        status=status.HTTP_405_METHOD_NOT_ALLOWED)


@api_view(['POST', ])
def reset_password(request):
    """view to reset the password"""
    if request.method == "POST":
        valid, errors = validate(*['code', 'password'], **get_request_data(request))
        if not valid:
            return Response({"error": errors}, status=status.HTTP_422_UNPROCESSABLE_ENTITY)
        data = get_request_data(request)
        hex_code = hexify(data.get("code"))
        try:
            forgot_password_model = ForgotPasswordModel.objects.get(code=hex_code)
        except ForgotPasswordModel.DoesNotExist:
            return Response({"error": ['code invalid']}, status=status.HTTP_404_NOT_FOUND)
        user = forgot_password_model.user
        user.set_password(data.get('password'))
        user.save(update_fields=("password",))
        forgot_password_model.delete()
        return Response({"message": "Password reset successfully"}, status=status.HTTP_202_ACCEPTED)
    else:
        return Response({'error': ['{} method is not allowed'.format(request.method)]},
                        status=status.HTTP_405_METHOD_NOT_ALLOWED)


@api_view(['POST', ])
@authentication_classes([TokenAuthentication, ])
@permission_classes([IsAuthenticated, ])
def change_password(request):
    """view to change password of the user"""
    if request.method == "POST":
        valid, errors = validate(*['current_password', 'new_password'], **get_request_data(request))
        if not valid:
            return Response({"error": errors}, status=status.HTTP_422_UNPROCESSABLE_ENTITY)
        data = get_request_data(request)
        user = request.user
        if user.check_password(data.get("current_password")):
            user.set_password(data.get('new_password'))
            user.save(update_fields=('password',))
            if SEND_REGISTRATION_MAILS:
                send_mail(
                    **{"body": "Hi {}!Your password changes successfully".format(user.username), "subject": "Password Changed",
                       "to_email": user.email})
            return Response({"message": "Password change successfully"}, status=status.HTTP_202_ACCEPTED)
        else:
            return Response({"errors": ['Current password is not correct']}, status=status.HTTP_401_UNAUTHORIZED)
    else:
        return Response({'error': ['{} method is not allowed'.format(request.method)]},
                        status=status.HTTP_405_METHOD_NOT_ALLOWED)


@api_view(['POST', ])
def login(request):
    """function for login and return the token for the user"""
    username = get_request_data(request).get('username')
    password = get_request_data(request).get('password')
    valid, errors = validate('username', 'password', **{"username": username, "password": password})
    if not valid:
        return Response({"errors": errors}, status=status.HTTP_400_BAD_REQUEST)
    user = authenticate(**{'username': username, 'password': password})
    if user:
        token, _ = Token.objects.get_or_create(user=user)
        return Response({"token": token.key}, status=status.HTTP_200_OK)
    else:
        return Response({"errors": ['Wrong Credentials']}, status=status.HTTP_401_UNAUTHORIZED)


@api_view(['POST', ])
def registration_view(request):
    """view to implement the registration of thr user"""
    data = get_request_data(request)
    valid, errors = validate('username', 'password', 'email', **data)
    if not valid:
        return Response({'error': errors}, status=status.HTTP_422_UNPROCESSABLE_ENTITY)
    username = data.get('username')
    password = data.get('password')
    email = data.get('email')

    if User.objects.filter(email=email).exists():
        return Response({"error": "User already registered with email"}, status=status.HTTP_409_CONFLICT)
    try:
        User.objects.create_user(username, email, password)
        if SEND_REGISTRATION_MAILS:
            send_mail(
                **{"body": "Hi {}! your registration is successful".format(username), "subject": "Registration Successful",
                   "to_email": email})
    except IntegrityError as e:
        error_text = str(e)
        error_field = error_text.split(".")[-1]
        return Response({"error":["user is already registered with {}".format(error_field)]},status=status.HTTP_409_CONFLICT)
    return Response({"message": [
        'User registered successfully'
    ]}, status=status.HTTP_201_CREATED)



from django.db import models
from django.contrib.auth.models import User

# Create your models here.
from .fields import SHA1Field
import hashlib


class ForgotPasswordModel(models.Model):
    """model for storing the reset password codes"""
    user = models.ForeignKey(User,on_delete=models.CASCADE)
    code = SHA1Field()
        
        